@@ -1,26 +1,32 @@
 #ifndef DNSSDINT_H
 #define DNSSDINT_H
 
+#include "global.h"
 #define RAOP_TXTVERS "1"
 #define RAOP_CH "2"             /* Audio channels: 2 */
-#define RAOP_CN "0,1,2,3"           /* Audio codec: PCM, ALAC, AAC, AAC ELD */
-#define RAOP_ET "0,3,5"           /* Encryption type: None, FairPlay, FairPlay SAPv2.5 */
+#define RAOP_CN "0,1,2,3"       /* Audio codec: PCM, ALAC, AAC, AAC ELD */
+#define RAOP_ET "0,3,5"         /* Encryption type: None, FairPlay, FairPlay SAPv2.5 */
 #define RAOP_VV "2"
-#define RAOP_FT "0x5A7FFEE6"
+#define FEATURES_1 "0x5A7FFEE6" /* first 32 bits of features */
+#define FEATURES_2 "0x0"        /* second 32 bits of features */
+#define RAOP_FT FEATURES_1 "," FEATURES_2
 #define RAOP_RHD "5.6.0.0"
 #define RAOP_SF "0x4"
 #define RAOP_SV "false"
 #define RAOP_DA "true"
 #define RAOP_SR "44100"         /* Sample rate: 44100 */
 #define RAOP_SS "16"            /* Sample size: 16 */
-#define RAOP_VS "220.68"
+#define RAOP_VS GLOBAL_VERSION  /* defined in global.h */
 #define RAOP_TP "UDP"           /* Transport protocol. Possible values: UDP or TCP or TCP,UDP */
 #define RAOP_MD "0,1,2"         /* Metadata: text, artwork, progress */
 #define RAOP_VN "65537"
 #define RAOP_PK "b07727d6f6cd6e08b58ede525ec3cdeaa252ad9f683feb212ef8a205246554e7"
 
-#define AIRPLAY_FEATURES "0x5A7FFEE6"
-#define AIRPLAY_SRCVERS "220.68"
+/* use same features for RAOP and AIRPLAY: is this correct? */
+#define AIRPLAY_FEATURES_1 FEATURES_1
+#define AIRPLAY_FEATURES_2 FEATURES_2
+#define AIRPLAY_FEATURES  AIRPLAY_FEATURES_1 "," AIRPLAY_FEATURES_2 
+#define AIRPLAY_SRCVERS GLOBAL_VERSION /*defined in global.h */
 #define AIRPLAY_FLAGS "0x4"
 #define AIRPLAY_VV "2"
 #define AIRPLAY_PK "b07727d6f6cd6e08b58ede525ec3cdeaa252ad9f683feb212ef8a205246554e7"
