@@ -19,7 +19,6 @@
 
 #include "raop.h"
 #include "raop_rtp.h"
-#include "raop_rtp.h"
 #include "pairing.h"
 #include "httpd.h"
 
@@ -51,12 +50,17 @@ struct raop_s {
     unsigned short data_lport;
     unsigned short mirror_data_lport;  
 
-    /* configurable plist items: width, height, refreshRate, maxFPS, overscanned */
+    /* configurable plist items: width, height, refreshRate, maxFPS, overscanned *
+     * also clientFPSdata, which controls whether video stream info received     *
+     * from the client is shown on terminal monitor.                             */
     uint16_t width;
     uint16_t height;
     uint8_t refreshRate;
     uint8_t maxFPS;
     uint8_t overscanned;
+    uint8_t clientFPSdata;
+
+    int max_ntp_timeouts;
 };
 
 struct raop_conn_s {
@@ -91,6 +95,7 @@ conn_init(void *opaque, unsigned char *local, int locallen, unsigned char *remot
     }
     conn->raop = raop;
     conn->raop_rtp = NULL;
+    conn->raop_rtp_mirror = NULL;
     conn->raop_ntp = NULL;
     conn->fairplay = fairplay_init(raop->logger);
 
@@ -181,6 +186,7 @@ conn_request(void *ptr, http_request_t *request, http_response_t **response) {
                     plist_to_xml(req_root_node, &plist_xml, &plist_len);
                     logger_log(conn->raop->logger, LOGGER_DEBUG, "%s", plist_xml);
                     free(plist_xml);
+                    plist_free(req_root_node);
                 } else if (data_is_text) {
                     char *data_str = utils_data_to_text((char *) request_data, request_datalen);
                     logger_log(conn->raop->logger, LOGGER_DEBUG, "%s", data_str);                    
@@ -198,7 +204,7 @@ conn_request(void *ptr, http_request_t *request, http_response_t **response) {
 
     http_response_add_header(*response, "CSeq", cseq);
     //http_response_add_header(*response, "Apple-Jack-Status", "connected; type=analog");
-    http_response_add_header(*response, "Server", "AirTunes/220.68");
+    http_response_add_header(*response, "Server", "AirTunes/"GLOBAL_VERSION);
 
     logger_log(conn->raop->logger, LOGGER_DEBUG, "Handling request %s with URL %s", method, url);
     raop_handler_t handler = NULL;
@@ -273,22 +279,34 @@ conn_request(void *ptr, http_request_t *request, http_response_t **response) {
                 }
 	    }
         }
+        plist_free(req_root_node);
         if (conn->raop->callbacks.conn_teardown) {
              conn->raop->callbacks.conn_teardown(conn->raop->callbacks.cls, &teardown_96, &teardown_110);
         }
         logger_log(conn->raop->logger, LOGGER_DEBUG, "TEARDOWN request,  96=%d, 110=%d", teardown_96, teardown_110);
 
-        //http_response_add_header(*response, "Connection", "close");
+        http_response_add_header(*response, "Connection", "close");
 
-        if (conn->raop_rtp != NULL && raop_rtp_is_running(conn->raop_rtp)) {
-	    /* Stop our RTP sessions */
-            raop_rtp_stop(conn->raop_rtp);
-        } else if (conn->raop_rtp_mirror) {
+        if (teardown_96) {
+            if (conn->raop_rtp) {
+	        /* Stop our audio RTP session */
+                raop_rtp_stop(conn->raop_rtp);
+            }
+        } else if (teardown_110) {
+            if (conn->raop_rtp_mirror) {
+                /* Stop our video RTP session */
+                raop_rtp_mirror_stop(conn->raop_rtp_mirror);
+            }
+        } else {
             /* Destroy our sessions */
-            raop_rtp_destroy(conn->raop_rtp);
-            conn->raop_rtp = NULL;
-            raop_rtp_mirror_destroy(conn->raop_rtp_mirror);
-            conn->raop_rtp_mirror = NULL;
+            if (conn->raop_rtp) {
+                raop_rtp_destroy(conn->raop_rtp);
+                conn->raop_rtp = NULL;
+            }
+            if (conn->raop_rtp_mirror) {
+                raop_rtp_mirror_destroy(conn->raop_rtp_mirror);
+                conn->raop_rtp_mirror = NULL;
+            }
         }
     }
     if (handler != NULL) {
@@ -318,6 +336,7 @@ conn_request(void *ptr, http_request_t *request, http_response_t **response) {
                 char * plist_xml;
                 uint32_t plist_len;
                 plist_to_xml(res_root_node, &plist_xml, &plist_len);
+                plist_free(res_root_node);
                 logger_log(conn->raop->logger, LOGGER_DEBUG, "%s", plist_xml);
                 free(plist_xml);
             } else if (data_is_text) {
@@ -346,9 +365,6 @@ conn_destroy(void *ptr) {
         conn->raop->callbacks.conn_destroy(conn->raop->callbacks.cls);
     }
 
-    if (conn->raop_ntp) {
-        raop_ntp_destroy(conn->raop_ntp);
-    }
     if (conn->raop_rtp) {
         /* This is done in case TEARDOWN was not called */
         raop_rtp_destroy(conn->raop_rtp);
@@ -357,7 +373,10 @@ conn_destroy(void *ptr) {
         /* This is done in case TEARDOWN was not called */
         raop_rtp_mirror_destroy(conn->raop_rtp_mirror);
     }
-    
+    if (conn->raop_ntp) {
+        raop_ntp_destroy(conn->raop_ntp);
+    }
+
     if (conn->raop->callbacks.video_flush) {
         conn->raop->callbacks.video_flush(conn->raop->callbacks.cls);
     }
@@ -437,7 +456,12 @@ raop_init(int max_clients, raop_callbacks_t *callbacks) {
     raop->refreshRate = 60;
     raop->maxFPS = 30;
     raop->overscanned = 0;
-    
+
+    /* initialize switch for display of client's streaming data records */    
+    raop->clientFPSdata = 0;
+
+    raop->max_ntp_timeouts = 0;
+
     return raop;
 }
 
@@ -474,21 +498,27 @@ int raop_set_plist(raop_t *raop, const char *plist_item, const int value) {
     assert(raop);
     assert(plist_item);
     
-    if (strcmp(plist_item,"width") == 0) {
+    if (strcmp(plist_item, "width") == 0) {
         raop->width = (uint16_t) value;
         if ((int) raop->width != value) retval = 1;
-    } else if (strcmp(plist_item,"height") == 0) {
+    } else if (strcmp(plist_item, "height") == 0) {
         raop->height = (uint16_t) value;
         if ((int) raop->height != value) retval = 1;
-    } else if (strcmp(plist_item,"refreshRate") == 0) {
+    } else if (strcmp(plist_item, "refreshRate") == 0) {
         raop->refreshRate = (uint8_t) value;
         if ((int) raop->refreshRate != value) retval = 1;
-    } else if (strcmp(plist_item,"maxFPS") == 0) {
+    } else if (strcmp(plist_item, "maxFPS") == 0) {
         raop->maxFPS = (uint8_t) value;
         if ((int) raop->maxFPS != value) retval = 1;
-    } else if (strcmp(plist_item,"overscanned") == 0) {
-      raop->overscanned = (uint8_t) (value ? 1 : 0);
-      if ((int) raop->overscanned  != value) retval = 1;
+    } else if (strcmp(plist_item, "overscanned") == 0) {
+        raop->overscanned = (uint8_t) (value ? 1 : 0);
+        if ((int) raop->overscanned  != value) retval = 1;
+    } else if (strcmp(plist_item, "clientFPSdata") == 0) {
+        raop->clientFPSdata = (value ? 1 : 0);
+        if ((int) raop->clientFPSdata  != value) retval = 1;
+    } else if (strcmp(plist_item, "max_ntp_timeouts") == 0) {
+        raop->max_ntp_timeouts = (value > 0 ? value : 0);
+        if (raop->max_ntp_timeouts != value) retval = 1;
     }  else {
         retval = -1;
     }	  
