@@ -28,6 +28,7 @@
 #include "stream.h"
 #include "global.h"
 #include "utils.h"
+#include "byteutils.h"
 
 #define RAOP_BUFFER_LENGTH 32
 
@@ -152,8 +153,14 @@ raop_buffer_decrypt(raop_buffer_t *raop_buffer, unsigned char *data, unsigned ch
 #endif
 
     if (DECRYPTION_TEST) {
-        printf("encrypted 12 byte header %s", utils_data_to_string(data,12,12));
-        if (payload_size) printf("len %d before decryption:\n%s", payload_size, utils_data_to_string(&data[12],16,16 ));
+        char *str = utils_data_to_string(data,12,12);
+        printf("encrypted 12 byte header %s", str);
+        free(str);
+        if (payload_size) {
+            str = utils_data_to_string(&data[12],16,16);
+            printf("len %d before decryption:\n%s", payload_size, str);
+            free(str);
+        }
     }
     encryptedlen = payload_size / 16*16;
     memset(output, 0, payload_size);
@@ -179,10 +186,14 @@ raop_buffer_decrypt(raop_buffer_t *raop_buffer, unsigned char *data, unsigned ch
         }
         if (DECRYPTION_TEST == 2) {
             printf("decrypted audio frame, len = %d\n", *outputlen);
-	    printf("%s",utils_data_to_string(output,payload_size,16));
+            char *str = utils_data_to_string(output,payload_size,16);
+	    printf("%s",str);
             printf("\n");
+            free(str);
         } else {
-            printf("%d after  \n%s\n", payload_size, utils_data_to_string(output,16,16 ));
+            char *str = utils_data_to_string(output,16,16);
+            printf("%d after  \n%s\n", payload_size, str);
+            free(str);
         }
     }
 #ifdef DUMP_AUDIO
@@ -197,13 +208,15 @@ raop_buffer_decrypt(raop_buffer_t *raop_buffer, unsigned char *data, unsigned ch
 
 int
 raop_buffer_enqueue(raop_buffer_t *raop_buffer, unsigned char *data, unsigned short datalen, uint64_t timestamp, int use_seqnum) {
+    unsigned char empty_packet_marker[] = { 0x00, 0x68, 0x34, 0x00 };
     assert(raop_buffer);
 
     /* Check packet data length is valid */
     if (datalen < 12 || datalen > RAOP_PACKET_LEN) {
         return -1;
     }
-    if (datalen == 16 && data[12] == 0x0 && data[13] == 0x68 && data[14] == 0x34 && data[15] == 0x0) {
+    /* before time is synchronized, some empty data packets are sent */
+    if (datalen == 16 && !memcmp(&data[12], empty_packet_marker, 4)) {
         return 0;
     }
     int payload_size = datalen - 12;
@@ -211,7 +224,7 @@ raop_buffer_enqueue(raop_buffer_t *raop_buffer, unsigned char *data, unsigned sh
     /* Get correct seqnum for the packet */
     unsigned short seqnum;
     if (use_seqnum) {
-        seqnum = (data[2] << 8) | data[3];
+        seqnum = byteutils_get_short_be(data, 2);
     } else {
         seqnum = raop_buffer->first_seqnum;
     }
@@ -256,7 +269,7 @@ raop_buffer_enqueue(raop_buffer_t *raop_buffer, unsigned char *data, unsigned sh
 }
 
 void *
-raop_buffer_dequeue(raop_buffer_t *raop_buffer, unsigned int *length, uint64_t *timestamp, int no_resend) {
+raop_buffer_dequeue(raop_buffer_t *raop_buffer, unsigned int *length, uint64_t *timestamp, unsigned short *seqnum, int no_resend) {
     assert(raop_buffer);
 
     /* Calculate number of entries in the current buffer */
@@ -289,6 +302,7 @@ raop_buffer_dequeue(raop_buffer_t *raop_buffer, unsigned int *length, uint64_t *
 
     /* Return entry payload buffer */
     *timestamp = entry->timestamp;
+    *seqnum = entry->seqnum;
     *length = entry->payload_size;
     entry->payload_size = 0;
     void* data = entry->payload_data;
@@ -302,7 +316,8 @@ void raop_buffer_handle_resends(raop_buffer_t *raop_buffer, raop_resend_cb_t res
 
     if (seqnum_cmp(raop_buffer->first_seqnum, raop_buffer->last_seqnum) < 0) {
         int seqnum, count;
-
+        logger_log(raop_buffer->logger, LOGGER_DEBUG, "raop_buffer_handle_resends first_seqnum=%u last seqnum=%u",
+                   raop_buffer->first_seqnum, raop_buffer->last_seqnum);
         for (seqnum = raop_buffer->first_seqnum; seqnum_cmp(seqnum, raop_buffer->last_seqnum) < 0; seqnum++) {
             raop_buffer_entry_t *entry = &raop_buffer->entries[seqnum % RAOP_BUFFER_LENGTH];
             if (entry->filled) {
@@ -323,6 +338,7 @@ void raop_buffer_flush(raop_buffer_t *raop_buffer, int next_seq) {
     for (int i = 0; i < RAOP_BUFFER_LENGTH; i++) {
         if (raop_buffer->entries[i].payload_data) {
             free(raop_buffer->entries[i].payload_data);
+            raop_buffer->entries[i].payload_data = NULL;   
             raop_buffer->entries[i].payload_size = 0;
         }
         raop_buffer->entries[i].filled = 0;
