@@ -47,12 +47,15 @@ raop_handler_info(raop_conn_t *conn,
     int pk_len = 0;
     char *pk = utils_parse_hex(AIRPLAY_PK, strlen(AIRPLAY_PK), &pk_len);
 
+    uint64_t features = ((uint64_t) strtoul(AIRPLAY_FEATURES_2, NULL, 16)) << 32;
+    features += (uint64_t) strtoul(AIRPLAY_FEATURES_1, NULL, 16);
+
     plist_t r_node = plist_new_dict();
 
     plist_t txt_airplay_node = plist_new_data(airplay_txt, airplay_txt_len);
     plist_dict_set_item(r_node, "txtAirPlay", txt_airplay_node);
 
-    plist_t features_node = plist_new_uint((uint64_t) 0x1E << 32 | 0x5A7FFFF7);
+    plist_t features_node = plist_new_uint(features);
     plist_dict_set_item(r_node, "features", features_node);
 
     plist_t name_node = plist_new_string(name);
@@ -89,7 +92,7 @@ raop_handler_info(raop_conn_t *conn,
     plist_t keep_alive_low_power_node = plist_new_uint(1);
     plist_dict_set_item(r_node, "keepAliveLowPower", keep_alive_low_power_node);
 
-    plist_t source_version_node = plist_new_string(AIRPLAY_SRCVERS);
+    plist_t source_version_node = plist_new_string(GLOBAL_VERSION);
     plist_dict_set_item(r_node, "sourceVersion", source_version_node);
 
     plist_t pk_node = plist_new_data(pk, pk_len);
@@ -161,6 +164,7 @@ raop_handler_info(raop_conn_t *conn,
     plist_dict_set_item(r_node, "displays", displays_node);
 
     plist_to_bin(r_node, response_data, (uint32_t *) response_datalen);
+    plist_free(r_node);
     http_response_add_header(response, "Content-Type", "application/x-apple-binary-plist");
     free(pk);
     free(hw_addr);
@@ -305,8 +309,6 @@ raop_handler_setup(raop_conn_t *conn,
                    http_request_t *request, http_response_t *response,
                    char **response_data, int *response_datalen)
 {
-    unsigned short remote_cport = 0;
-
     const char *transport;
     int use_udp;
     const char *dacp_id;
@@ -351,6 +353,7 @@ raop_handler_setup(raop_conn_t *conn,
 
         unsigned char aesiv[16];
         unsigned char aeskey[16];
+        unsigned char eaeskey[72];
 
         logger_log(conn->raop->logger, LOGGER_DEBUG, "SETUP 1");
 
@@ -359,6 +362,7 @@ raop_handler_setup(raop_conn_t *conn,
         uint64_t eiv_len = 0;
         plist_get_data_val(req_eiv_node, &eiv, &eiv_len);
         memcpy(aesiv, eiv, 16);
+        free(eiv);	
         logger_log(conn->raop->logger, LOGGER_DEBUG, "eiv_len = %llu", eiv_len);
         char* str = utils_data_to_string(aesiv, 16, 16);
         logger_log(conn->raop->logger, LOGGER_DEBUG, "16 byte aesiv (needed for AES-CBC audio decryption iv):\n%s", str);
@@ -367,13 +371,15 @@ raop_handler_setup(raop_conn_t *conn,
         char* ekey = NULL;
         uint64_t ekey_len = 0;
         plist_get_data_val(req_ekey_node, &ekey, &ekey_len);
+        memcpy(eaeskey,ekey,72);
+        free(ekey);
         logger_log(conn->raop->logger, LOGGER_DEBUG, "ekey_len = %llu", ekey_len);
-        // ekey is 72 bytes, aeskey is 16 bytes
-        str = utils_data_to_string((unsigned char *) ekey, ekey_len, 16);
+        // eaeskey is 72 bytes, aeskey is 16 bytes
+        str = utils_data_to_string((unsigned char *) eaeskey, ekey_len, 16);
         logger_log(conn->raop->logger, LOGGER_DEBUG, "ekey:\n%s", str);
         free (str);
 
-        int ret = fairplay_decrypt(conn->fairplay, (unsigned char*) ekey, aeskey);
+        int ret = fairplay_decrypt(conn->fairplay, (unsigned char*) eaeskey, aeskey);
         logger_log(conn->raop->logger, LOGGER_DEBUG, "fairplay_decrypt ret = %d", ret);
         str = utils_data_to_string(aeskey, 16, 16);
         logger_log(conn->raop->logger, LOGGER_DEBUG, "16 byte aeskey (fairplay-decrypted from ekey):\n%s", str);
@@ -395,7 +401,6 @@ raop_handler_setup(raop_conn_t *conn,
         if  (old_protocol) {    /* some windows AirPlay-client emulators use old AirPlay 1 protocol with unhashed AES key */
             logger_log(conn->raop->logger, LOGGER_INFO, "Client identifed as using old protocol (unhashed) AES audio key)");
         } else {
-            unsigned char eaeskey[64] = {};
             memcpy(eaeskey, aeskey, 16);
             sha_ctx_t *ctx = sha_init();
             sha_update(ctx, eaeskey, 16);
@@ -415,8 +420,8 @@ raop_handler_setup(raop_conn_t *conn,
         logger_log(conn->raop->logger, LOGGER_DEBUG, "timing_rport = %llu", timing_rport);
 
         unsigned short timing_lport = conn->raop->timing_lport;
-        conn->raop_ntp = raop_ntp_init(conn->raop->logger, conn->remote, conn->remotelen, timing_rport);
-        raop_ntp_start(conn->raop_ntp, &timing_lport);
+        conn->raop_ntp = raop_ntp_init(conn->raop->logger, &conn->raop->callbacks, conn->remote, conn->remotelen, timing_rport);
+        raop_ntp_start(conn->raop_ntp, &timing_lport, conn->raop->max_ntp_timeouts);
 
         conn->raop_rtp = raop_rtp_init(conn->raop->logger, &conn->raop->callbacks, conn->raop_ntp, conn->remote, conn->remotelen, aeskey, aesiv);
         conn->raop_rtp_mirror = raop_rtp_mirror_init(conn->raop->logger, &conn->raop->callbacks, conn->raop_ntp, conn->remote, conn->remotelen, aeskey);
@@ -452,8 +457,8 @@ raop_handler_setup(raop_conn_t *conn,
                     logger_log(conn->raop->logger, LOGGER_DEBUG, "streamConnectionID (needed for AES-CTR video decryption key and iv): %llu", stream_connection_id);
 
                     if (conn->raop_rtp_mirror) {
-                        raop_rtp_init_mirror_aes(conn->raop_rtp_mirror, stream_connection_id);
-                        raop_rtp_start_mirror(conn->raop_rtp_mirror, use_udp, &dport);
+                        raop_rtp_init_mirror_aes(conn->raop_rtp_mirror, &stream_connection_id);
+                        raop_rtp_start_mirror(conn->raop_rtp_mirror, use_udp, &dport, conn->raop->clientFPSdata);
                         logger_log(conn->raop->logger, LOGGER_DEBUG, "Mirroring initialized successfully");
                     } else {
                         logger_log(conn->raop->logger, LOGGER_ERR, "Mirroring not initialized at SETUP, playing will fail!");
@@ -471,40 +476,43 @@ raop_handler_setup(raop_conn_t *conn,
                 } case 96: {
                     // Audio
                     unsigned short cport = conn->raop->control_lport, dport = conn->raop->data_lport; 
+                    unsigned short remote_cport = 0;
+                    unsigned char ct;
+                    uint64_t uint_val = 0;
+                    plist_t req_stream_control_port_node = plist_dict_get_item(req_stream_node, "controlPort");
+                    plist_get_uint_val(req_stream_control_port_node, &uint_val);
+                    remote_cport = (unsigned short) uint_val;   /* must != 0 to activate audio resend requests */
 
-                    uint64_t audioFormat;
-		    unsigned char ct;
-	            unsigned short spf;
-                    bool isMedia; 
-                    bool usingScreen;
+                    plist_t req_stream_ct_node = plist_dict_get_item(req_stream_node, "ct");
+                    plist_get_uint_val(req_stream_ct_node, &uint_val);
+                    ct = (unsigned char) uint_val;
 
                     if (conn->raop->callbacks.audio_get_format) {
-		        /* get audio compression type */
-		        uint64_t uint_val;
-                        uint8_t bool_val;
-			
-                        plist_t req_stream_ct_node = plist_dict_get_item(req_stream_node, "ct");
-                        plist_get_uint_val(req_stream_ct_node, &uint_val);
-                        ct = (unsigned char) uint_val;
+		        /* get additional audio format parameters  */
+                        uint64_t audioFormat;
+                        unsigned short spf;
+                        bool isMedia; 
+                        bool usingScreen;
+                        uint8_t bool_val = 0;
 
                         plist_t req_stream_spf_node = plist_dict_get_item(req_stream_node, "spf");
                         plist_get_uint_val(req_stream_spf_node, &uint_val);
                         spf = (unsigned short) uint_val;
-		    
+
                         plist_t req_stream_audio_format_node = plist_dict_get_item(req_stream_node, "audioFormat");
                         plist_get_uint_val(req_stream_audio_format_node, &audioFormat);
-			
-                        plist_t req_stream_ismedia_node = plist_dict_get_item(req_stream_node, "isMedia");
-		        if (req_stream_ismedia_node) {
-		            plist_get_bool_val(req_stream_ismedia_node, &bool_val);
-			    isMedia = (bool) bool_val;
+
+                        plist_t req_stream_is_media_node = plist_dict_get_item(req_stream_node, "isMedia");
+                        if (req_stream_is_media_node) {
+                            plist_get_bool_val(req_stream_is_media_node, &bool_val);
+                            isMedia = (bool) bool_val;
                         } else {
                             isMedia = false;
                         }
 
-                        plist_t req_stream_usingscreen_node = plist_dict_get_item(req_stream_node, "usingScreen");
-                        if (req_stream_usingscreen_node) {
-                            plist_get_bool_val(req_stream_usingscreen_node, &bool_val);
+                        plist_t req_stream_using_screen_node = plist_dict_get_item(req_stream_node, "usingScreen");
+                        if (req_stream_using_screen_node) {
+                            plist_get_bool_val(req_stream_using_screen_node, &bool_val);
                             usingScreen = (bool) bool_val;
                         } else {
                             usingScreen = false;
@@ -514,7 +522,7 @@ raop_handler_setup(raop_conn_t *conn,
                     }
 
                     if (conn->raop_rtp) {
-                        raop_rtp_start_audio(conn->raop_rtp, use_udp, remote_cport, &cport, &dport);
+                        raop_rtp_start_audio(conn->raop_rtp, use_udp, remote_cport, &cport, &dport, ct);
                         logger_log(conn->raop->logger, LOGGER_DEBUG, "RAOP initialized success");
                     } else {
                         logger_log(conn->raop->logger, LOGGER_ERR, "RAOP not initialized at SETUP, playing will fail!");
@@ -544,6 +552,8 @@ raop_handler_setup(raop_conn_t *conn,
     }
 
     plist_to_bin(res_root_node, response_data, (uint32_t*) response_datalen);
+    plist_free(res_root_node);
+    plist_free(req_root_node);
     http_response_add_header(response, "Content-Type", "application/x-apple-binary-plist");
 }
 
