@@ -28,15 +28,23 @@
 #include "byteutils.h"
 #include "mirror_buffer.h"
 #include "stream.h"
+#include "utils.h"
 
 #define NO_FLUSH (-42)
 
 #define RAOP_RTP_SAMPLE_RATE (44100.0 / 1000000.0)
 #define RAOP_RTP_SYNC_DATA_COUNT 8
+#define SEC 1000000
+
+#define DELAY_AAC 500000 //empirical, matches audio latency of about -0.5 sec after first clock sync event
+#define DELAY_ALAC 2000000 //empirical, matches audio latency of about -2.0 sec after first clock sync event
+
+/* note: it is unclear what will happen in the unlikely event that this code is running at the time of the unix-time 
+ * epoch event on 2038-01-19 at 3:14:08 UTC ! (but Apple will surely have removed AirPlay "legacy pairing" by then!) */
 
 typedef struct raop_rtp_sync_data_s {
-    uint64_t ntp_time; // The local wall clock time at the time of rtp_time
-    uint32_t rtp_time; // The remote rtp clock time corresponding to ntp_time
+    uint64_t ntp_time;  // The local wall clock time (unix time in usec) at the time of rtp_time
+    uint64_t rtp_time;   // The remote rtp clock time corresponding to ntp_time
 } raop_rtp_sync_data_t;
 
 struct raop_rtp_s {
@@ -49,6 +57,10 @@ struct raop_rtp_s {
     int64_t rtp_sync_offset;
     raop_rtp_sync_data_t sync_data[RAOP_RTP_SYNC_DATA_COUNT];
     int sync_data_index;
+    uint64_t ntp_start_time;
+    uint64_t rtp_start_time;
+    uint64_t rtp_time;
+    bool rtp_clock_started;
 
     // Transmission Stats, could be used if a playout buffer is needed
     // float interarrival_jitter; // As defined by RTP RFC 3550, Section 6.4.1
@@ -102,6 +114,9 @@ struct raop_rtp_s {
     struct sockaddr_storage control_saddr;
     socklen_t control_saddr_len;
     unsigned short control_seqnum;
+
+    /* audio compression type: ct = 2 (ALAC), ct = 8 (AAC_ELD) (ct = 4 would be AAC-MAIN) */
+    unsigned char ct;
 };
 
 static int
@@ -154,6 +169,15 @@ raop_rtp_init(logger_t *logger, raop_callbacks_t *callbacks, raop_ntp_t *ntp, co
         raop_rtp->sync_data[i].ntp_time = 0;
         raop_rtp->sync_data[i].rtp_time = 0;
     }
+    raop_rtp->ntp_start_time = 0;
+    raop_rtp->rtp_start_time = 0;
+    raop_rtp->rtp_clock_started = false;
+
+    
+    raop_rtp->dacp_id = NULL;
+    raop_rtp->active_remote_header = NULL;
+    raop_rtp->metadata = NULL;
+    raop_rtp->coverart = NULL;
 
     memcpy(&raop_rtp->callbacks, callbacks, sizeof(raop_callbacks_t));
     raop_rtp->buffer = raop_buffer_init(logger, aeskey, aesiv);
@@ -364,28 +388,58 @@ raop_rtp_process_events(raop_rtp_t *raop_rtp, void *cb_data)
     return 0;
 }
 
-void raop_rtp_sync_clock(raop_rtp_t *raop_rtp, uint32_t rtp_time, uint64_t ntp_time) {
-    raop_rtp->sync_data_index = (raop_rtp->sync_data_index + 1) % RAOP_RTP_SYNC_DATA_COUNT;
-    raop_rtp->sync_data[raop_rtp->sync_data_index].rtp_time = rtp_time;
-    raop_rtp->sync_data[raop_rtp->sync_data_index].ntp_time = ntp_time;
-
+void raop_rtp_sync_clock(raop_rtp_t *raop_rtp, uint64_t ntp_time, uint64_t rtp_time,  int shift) {
+    int latest;
     uint32_t valid_data_count = 0;
     valid_data_count = 0;
-    int64_t total_offsets = 0;
-    for (int i = 0; i < RAOP_RTP_SYNC_DATA_COUNT; ++i) {
+    double total_offsets = 0;
+    int64_t rtp_offset, avg_offset, correction;
+
+    raop_rtp->sync_data_index = (raop_rtp->sync_data_index + 1) % RAOP_RTP_SYNC_DATA_COUNT;
+    latest = raop_rtp->sync_data_index;
+    raop_rtp->sync_data[latest].rtp_time = rtp_time;
+    raop_rtp->sync_data[latest].ntp_time = ntp_time;
+
+    for (int i = 0; i < RAOP_RTP_SYNC_DATA_COUNT; i++) {
         if (raop_rtp->sync_data[i].ntp_time == 0) continue;
-        total_offsets += (int64_t) (((double) raop_rtp->sync_data[i].rtp_time) / raop_rtp->rtp_sync_scale) - raop_rtp->sync_data[i].ntp_time;
+        rtp_offset = ((int64_t) raop_rtp->sync_data[i].rtp_time) - ((int64_t) raop_rtp->sync_data[latest].rtp_time);
+        total_offsets += ((double) rtp_offset) / raop_rtp-> rtp_sync_scale;
+        total_offsets -= (double) (((int64_t) raop_rtp->sync_data[i].ntp_time) - ((int64_t) raop_rtp->sync_data[latest].ntp_time));
         valid_data_count++;
     }
-    int64_t avg_offset = total_offsets / valid_data_count;
-    int64_t correction = avg_offset - raop_rtp->rtp_sync_offset;
+    total_offsets = (total_offsets / valid_data_count);
+    rtp_offset =  ((int64_t) raop_rtp->sync_data[latest].rtp_time) - ((int64_t) raop_rtp->rtp_start_time) + ((int64_t) shift);
+    total_offsets +=  ((double) rtp_offset) / raop_rtp->rtp_sync_scale;
+    avg_offset = (int64_t) total_offsets;
+    avg_offset -= ((int64_t) raop_rtp->sync_data[latest].ntp_time) - ((int64_t) raop_rtp->ntp_start_time);
+    correction = avg_offset - raop_rtp->rtp_sync_offset;
     raop_rtp->rtp_sync_offset = avg_offset;
-
-    logger_log(raop_rtp->logger, LOGGER_DEBUG, "raop_rtp sync correction=%lld", correction);
+    logger_log(raop_rtp->logger, LOGGER_DEBUG, "raop_rtp sync correction=%lld, rtp_sync_offset = %8.6f ",
+               correction, ((double) raop_rtp->rtp_sync_offset) / SEC);
 }
 
-uint64_t raop_rtp_convert_rtp_time(raop_rtp_t *raop_rtp, uint32_t rtp_time) {
-    return (uint64_t) (((double) rtp_time) / raop_rtp->rtp_sync_scale) - raop_rtp->rtp_sync_offset;
+uint64_t rtp64_time (raop_rtp_t *raop_rtp, const uint32_t *rtp32) {
+    /* convert from 32-bit to 64-bit rtp time:
+     * the rtp_time 32-bit epoch at 44.1kHz has length of about 27 hours
+     * using 64-bit rtp time avoids any epoch issues.
+     * initial call sets epoch to 1; subsequent calls maintain consistent epoch.
+     * (assumes successive calls are close in time) */
+
+    if (raop_rtp->rtp_clock_started) {
+        uint32_t diff1 = *rtp32 -  ((uint32_t) raop_rtp->rtp_time);
+        uint32_t diff2 = ((uint32_t) raop_rtp->rtp_time) - *rtp32;
+        if (diff1 <= diff2) {
+            raop_rtp->rtp_time += (uint64_t) diff1;
+        } else {
+            raop_rtp->rtp_time -= (uint64_t) diff2;
+        }
+    } else {
+        raop_rtp->rtp_time = (0x01ULL << 32 ) + (uint64_t) *rtp32;
+        raop_rtp->rtp_start_time = raop_rtp->rtp_time;
+        raop_rtp->rtp_clock_started = true;
+    }
+    //assert(*rtp32 == (uint32_t) raop_rtp->rtp_time);
+    return  raop_rtp->rtp_time;
 }
 
 static THREAD_RETVAL
@@ -396,13 +450,30 @@ raop_rtp_thread_udp(void *arg)
     unsigned int packetlen;
     struct sockaddr_storage saddr;
     socklen_t saddrlen;
+
+    /* for initial rtp to ntp conversions */    
+    bool have_synced = false;
+    int rtp_count = 0;
+    int64_t initial_offset = 0;
+    double sync_adjustment = 0;
+    int64_t delay = 0;
+    unsigned short seqnum1 = 0, seqnum2 = 0;
+    bool offset_estimate_initialized  = false;
+
     assert(raop_rtp);
+    raop_rtp->ntp_start_time = raop_ntp_get_local_time(raop_rtp->ntp);
+    raop_rtp->rtp_clock_started = false;
+    for (int i = 0; i < RAOP_RTP_SYNC_DATA_COUNT; i++) {
+        raop_rtp->sync_data[i].ntp_time = 0;
+    }
+
+    logger_log(raop_rtp->logger, LOGGER_DEBUG, "raop_rtp start_time = %8.6f (raop_rtp audio)",
+               ((double) raop_rtp->ntp_start_time) / SEC);
 
     while(1) {
         fd_set rfds;
         struct timeval tv;
-        int nfds, ret;
-
+        int nfds, ret;	
         /* Check if we are still running and process callbacks */
         if (raop_rtp_process_events(raop_rtp, NULL)) {
             break;
@@ -439,32 +510,103 @@ raop_rtp_thread_udp(void *arg)
             memcpy(&raop_rtp->control_saddr, &saddr, saddrlen);
             raop_rtp->control_saddr_len = saddrlen;
             int type_c = packet[1] & ~0x80;
-            logger_log(raop_rtp->logger, LOGGER_DEBUG, "raop_rtp type_c 0x%02x, packetlen = %d", type_c, packetlen);
-            if (type_c == 0x56) {
-                /* Handle resent data packet */
-                uint32_t rtp_timestamp =  (packet[4 + 4] << 24) | (packet[4 + 5] << 16) | (packet[4 + 6] << 8) | packet[4 + 7];
-                uint64_t ntp_timestamp = raop_rtp_convert_rtp_time(raop_rtp, rtp_timestamp);
-                uint64_t ntp_now = raop_ntp_get_local_time(raop_rtp->ntp);
-                logger_log(raop_rtp->logger, LOGGER_DEBUG, "raop_rtp audio resent: ntp = %llu, now = %llu, latency=%lld, rtp=%u",
-                           ntp_timestamp, ntp_now, ((int64_t) ntp_now) - ((int64_t) ntp_timestamp), rtp_timestamp);
-                int result = raop_buffer_enqueue(raop_rtp->buffer, packet + 4, packetlen - 4, ntp_timestamp, 1);
-                assert(result >= 0);
+            logger_log(raop_rtp->logger, LOGGER_DEBUG, "\nraop_rtp type_c 0x%02x, packetlen = %d", type_c, packetlen);
+
+            if (type_c == 0x56 && packetlen >= 8) {
+	        /* Handle resent data packet, which begins at offset 4 of these packets */
+                unsigned char *resent_packet =  &packet[4];
+                unsigned int resent_packetlen = packetlen - 4;
+                unsigned short seqnum = byteutils_get_short_be(resent_packet, 2);
+                if (resent_packetlen >= 12) {
+                    uint32_t timestamp = byteutils_get_int_be(resent_packet, 4);
+                    uint64_t rtp_time = rtp64_time(raop_rtp, &timestamp);
+                    logger_log(raop_rtp->logger, LOGGER_DEBUG, "raop_rtp resent audio packet: seqnum=%u", seqnum);
+                    assert(raop_buffer_enqueue(raop_rtp->buffer, resent_packet, resent_packetlen, rtp_time, 1) >= 0);
+                } else {
+                    /* type_c = 0x56 packets  with length 8 have been reported */
+                    char *str = utils_data_to_string(packet, packetlen, 16);
+                    logger_log(raop_rtp->logger, LOGGER_DEBUG, "Received empty resent audio packet length %d, seqnum=%u:\n%s",
+                               packetlen, seqnum, str);
+                    free (str);
+                }
             } else if (type_c == 0x54 && packetlen >= 20) {
+                /* packet[0] = 0x90 (first sync ?) or 0x80 (subsequent ones)
+                 * packet[1] = 0xd4,  (0xd4 && ~0x80 = type 0x54)
+                 * packet[2:3] = 0x00 0x04
+                 * packet[4:7] : sync_rtp (big-endian uint32_t)
+                 * packet[8:15]: remote ntp timestamp (big-endian uint64_t)  
+                 * packet[16:20]: next_rtp (big-endian uint32_t)
+                 * next_rtp = sync_rtp + 7497 =  441 *  17 (0.17 sec) for AAC-ELD
+                 * next_rtp = sync_rtp + 77175  = 441 * 175 (1.75 sec) for ALAC */
+
                 // The unit for the rtp clock is 1 / sample rate = 1 / 44100
-                uint32_t sync_rtp = byteutils_get_int_be(packet, 4) - 11025;
+                uint32_t sync_rtp = byteutils_get_int_be(packet, 4);
+                uint64_t sync_rtp64 = rtp64_time(raop_rtp, &sync_rtp);
+ 
+
+                if (have_synced == false) {
+                    logger_log(raop_rtp->logger, LOGGER_DEBUG, "first audio rtp sync");
+                    have_synced = true;
+                }
                 uint64_t sync_ntp_raw = byteutils_get_long_be(packet, 8);
                 uint64_t sync_ntp_remote = raop_ntp_timestamp_to_micro_seconds(sync_ntp_raw, true);
                 uint64_t sync_ntp_local = raop_ntp_convert_remote_time(raop_rtp->ntp, sync_ntp_remote);
-                // It's not clear what the additional rtp timestamp indicates
-                uint32_t next_rtp = byteutils_get_int_be(packet, 16);
-                logger_log(raop_rtp->logger, LOGGER_DEBUG, "raop_rtp sync: ntp=%llu, local ntp: %llu, rtp=%u, rtp_next=%u",
-                           sync_ntp_remote, sync_ntp_local, sync_rtp, next_rtp);
-                raop_rtp_sync_clock(raop_rtp, sync_rtp, sync_ntp_local);
+                int64_t shift;
+                switch (raop_rtp->ct) {
+                case 0x08:                   /*AAC-ELD */
+                    shift = -11025;           /* 44100/4 */
+                    break;
+                case 0x02:
+                default:
+                    shift = 0;   /* not needed for ALAC (audio only) */
+                    break;
+                }
+                char *str = utils_data_to_string(packet, packetlen, 20);
+                logger_log(raop_rtp->logger, LOGGER_DEBUG,
+                           "raop_rtp sync: client ntp=%8.6f, ntp = %8.6f, ntp_start_time %8.6f, sync_rtp=%u\n%s",
+                           ((double) sync_ntp_remote) / SEC, ((double)sync_ntp_local) / SEC,
+                           ((double) raop_rtp->ntp_start_time) / SEC, sync_rtp, str);
+                free(str);
+                raop_rtp_sync_clock(raop_rtp, sync_ntp_local, sync_rtp64, shift);		
             } else {
-                logger_log(raop_rtp->logger, LOGGER_DEBUG, "raop_rtp unknown packet");
+                char *str = utils_data_to_string(packet, packetlen, 16);
+                logger_log(raop_rtp->logger, LOGGER_DEBUG, "raop_rtp unknown udp control packet\n%s", str);
+                free(str);
             }
         }
 
+        /* rtp audio data packets:
+         * packet[0] 0x80
+         * packet[1] 0x60 = 96
+         * packet[2:3] seqnum (big-endian unsigned short)
+         * packet[4:7] rtp timestamp (big-endian unsigned int)
+         * packet[8:11] 0x00 0x00 0x00 0x00
+         * packet[12:packetlen - 1] encrypted audio payload
+         * For (AAC-ELD only), the payload of initial packets at the start of
+         * the stream may be replaced by a 4-byte "no_data_marker" 0x00 0x68 0x34 0x00 */
+	
+        /* consecutive AAC-ELD rtp timestamps differ by spf = 480
+         * consecutive ALAC rtp timestamps differ by spf = 352
+         * both have PCM uncompressed sampling rate = 441000 Hz */
+
+        /* clock time in microseconds advances at (rtp_timestamp * 1000000)/44100 between frames */
+
+        /* every AAC-ELD packet is sent three times:  0  0 1  0 1 2  1 2 3  2 3 4 ... 
+         * (after decoding AAC-ELD into PCM, the sound frame is three times bigger)
+         * ALAC packets are sent once only  0 1 2 3 4 5  ...  */
+
+        /* When the AAC-ELD audio stream starts, the initial packets are length-16 packets with
+         * a four-byte "no_data_marker" 0x00 0x68 0x34 0x00 replacing the payload.
+         * The 12-byte packetheader contains  a secnum and rtp_timestamp, and each  packets is sent
+         * three times; the secnum and rtp_timestamp increment according to the same pattern as 
+         * AAC-ELD packets with audio content.*/
+
+	 /* When the ALAC audio stream starts, the initial packets are length-44 packets with 
+	  * the same 32-byte encrypted payload which after decryption is the beginning of a
+          * 32-byte ALAC packet, presumably with format information, but not actual audio data.
+          * The secnum and rtp_timestamp in the packet header increment according to the same
+          * pattern as ALAC packets with audio content */
+
         if (FD_ISSET(raop_rtp->dsock, &rfds)) {
             //logger_log(raop_rtp->logger, LOGGER_INFO, "Would have data packet in queue");
             // Receiving audio data here
@@ -474,39 +616,84 @@ raop_rtp_thread_udp(void *arg)
             // rtp payload type
             //int type_d = packet[1] & ~0x80;
             //logger_log(raop_rtp->logger, LOGGER_DEBUG, "raop_rtp_thread_udp type_d 0x%02x, packetlen = %d", type_d, packetlen);
-
-            // Len = 16 appears if there is no time
             if (packetlen >= 12) {
-                int no_resend = (raop_rtp->control_rport == 0);// false
-
-                uint32_t rtp_timestamp =  (packet[4] << 24) | (packet[5] << 16) | (packet[6] << 8) | packet[7];
-                uint64_t ntp_timestamp = raop_rtp_convert_rtp_time(raop_rtp, rtp_timestamp);
-                uint64_t ntp_now = raop_ntp_get_local_time(raop_rtp->ntp);
-                logger_log(raop_rtp->logger, LOGGER_DEBUG, "raop_rtp audio: ntp = %llu, now = %llu, latency=%lld, rtp=%u",
-                           ntp_timestamp, ntp_now, ((int64_t) ntp_now) - ((int64_t) ntp_timestamp), rtp_timestamp);
-
-                int result = raop_buffer_enqueue(raop_rtp->buffer, packet, packetlen, ntp_timestamp, 1);
-                assert(result >= 0);
-
+                int no_resend = (raop_rtp->control_rport == 0); /* true when control_rport is not set */
+                uint32_t rtp_timestamp =  byteutils_get_int_be(packet, 4);
+                uint64_t rtp_time = rtp64_time(raop_rtp, &rtp_timestamp);
+                if (have_synced == false) {
+                    /* until the first rtp sync occurs, we don't know the exact client ntp timestamp that matches the client rtp timestamp */
+                    int64_t sync_ntp =  ((int64_t) raop_ntp_get_local_time(raop_rtp->ntp)) - ((int64_t) raop_rtp->ntp_start_time) ;
+                    int64_t sync_rtp = ((int64_t) rtp_time) - ((int64_t) raop_rtp->rtp_start_time);
+                    int64_t offset;
+                    unsigned short seqnum = byteutils_get_short_be(packet,2);
+                    if (!offset_estimate_initialized) {
+                        offset_estimate_initialized = true;
+                        switch (raop_rtp->ct) {
+                        case 0x02:  
+                            delay = DELAY_ALAC;   /* DELAY = 2000000 (2.0 sec) is empirical choice for ALAC */
+                            logger_log(raop_rtp->logger, LOGGER_DEBUG, "Audio is ALAC: using initial latency estimate -%8.6f sec",
+                                      ((double) delay) / SEC);
+                            break;
+                        case 0x08:
+                            delay = DELAY_AAC;   /* DELAY = 500000 (0.5 sec) is empirical choice for AAC-ELD */
+                            logger_log(raop_rtp->logger, LOGGER_DEBUG, "Audio is AAC: using initial latency estimate -%8.6f sec",
+                                       ((double) delay ) / SEC);
+                            break;
+                        default:
+                            break;
+                        }
+                        initial_offset = -(sync_ntp + delay);
+                        raop_rtp->rtp_sync_offset = initial_offset;
+                        sync_adjustment = 0;
+                        seqnum1 = seqnum;
+                        seqnum2 = seqnum;
+                    }
+                    sync_ntp += delay;
+                    offset = -sync_ntp;
+                    if (seqnum2 != seqnum) {  /* for AAC-ELD  only use copy 3 of the 3 copies of each  frame */
+                        rtp_count++;
+                        offset -= initial_offset;
+                        sync_adjustment += ((double) offset) + (((double) sync_rtp) / raop_rtp->rtp_sync_scale);
+                        raop_rtp->rtp_sync_offset = initial_offset + (int64_t) (sync_adjustment / rtp_count);
+                        //logger_log(raop_rtp->logger, LOGGER_DEBUG, "initial estimate of rtp_sync_offset %d secnum = %u:  %8.6f",
+                        //           rtp_count, seqnum,  ((double) raop_rtp->rtp_sync_offset) / SEC);
+		    }
+                    seqnum2 = seqnum1;
+                    seqnum1 = seqnum;
+                }
+                assert(raop_buffer_enqueue(raop_rtp->buffer, packet, packetlen, rtp_time, 1) >= 0);
                 // Render continuous buffer entries
                 void *payload = NULL;
                 unsigned int payload_size;
-                uint64_t timestamp;
-                while ((payload = raop_buffer_dequeue(raop_rtp->buffer, &payload_size, &timestamp, no_resend))) {
-                    aac_decode_struct aac_data;
-                    aac_data.data_len = payload_size;
-                    aac_data.data = payload;
-                    aac_data.pts = timestamp;
-                    raop_rtp->callbacks.audio_process(raop_rtp->callbacks.cls, raop_rtp->ntp, &aac_data);
+                unsigned short seqnum;
+                uint64_t rtp64_timestamp;
+                while ((payload = raop_buffer_dequeue(raop_rtp->buffer, &payload_size, &rtp64_timestamp, &seqnum, no_resend))) {
+                    double  elapsed_time =  (((double) (rtp64_timestamp - (uint64_t) raop_rtp->rtp_start_time)) / raop_rtp->rtp_sync_scale);
+                    audio_decode_struct audio_data; 
+                    audio_data.data_len = payload_size;
+                    audio_data.data = payload;
+                    audio_data.ntp_time = raop_rtp->ntp_start_time + (uint64_t) elapsed_time;
+                    audio_data.ntp_time -= raop_rtp->rtp_sync_offset;
+                    audio_data.rtp_time = rtp64_timestamp;
+                    audio_data.seqnum = seqnum;
+                    raop_rtp->callbacks.audio_process(raop_rtp->callbacks.cls, raop_rtp->ntp, &audio_data);
                     free(payload);
+                    uint64_t ntp_now = raop_ntp_get_local_time(raop_rtp->ntp);
+                    int64_t latency =  ((int64_t) ntp_now) - ((int64_t) audio_data.ntp_time); 
+                    logger_log(raop_rtp->logger, LOGGER_DEBUG, "raop_rtp audio: now = %8.6f, npt = %8.6f, latency = %8.6f, rtp_time=%u seqnum = %u",
+                               ((double) ntp_now ) / SEC, ((double) audio_data.ntp_time) / SEC, ((double) latency) / SEC, (uint32_t) rtp64_timestamp,
+                               seqnum);
                 }
 
                 /* Handle possible resend requests */
                 if (!no_resend) {
                     raop_buffer_handle_resends(raop_rtp->buffer, raop_rtp_resend_callback, raop_rtp);
                 }
+            } else {
+                   char *str = utils_data_to_string(packet, packetlen, 16);
+                   logger_log(raop_rtp->logger, LOGGER_DEBUG, "Received short type_d = 0x%2x  packet with length %d:\n%s", packet[1] & ~0x80, packetlen, str);
+                   free (str);
             }
-
         }
     }
 
@@ -523,7 +710,7 @@ raop_rtp_thread_udp(void *arg)
 // Start rtp service, three udp ports
 void
 raop_rtp_start_audio(raop_rtp_t *raop_rtp, int use_udp, unsigned short control_rport,
-                     unsigned short *control_lport, unsigned short *data_lport)
+                     unsigned short *control_lport, unsigned short *data_lport, unsigned char ct)
 {
     logger_log(raop_rtp->logger, LOGGER_INFO, "raop_rtp starting audio");
     int use_ipv6 = 0;
@@ -538,6 +725,8 @@ raop_rtp_start_audio(raop_rtp_t *raop_rtp, int use_udp, unsigned short control_r
         return;
     }
 
+    raop_rtp->ct = ct;
+
     /* Initialize ports and sockets */
     raop_rtp->control_lport = *control_lport;
     raop_rtp->data_lport = *data_lport;
@@ -632,7 +821,13 @@ raop_rtp_remote_control_id(raop_rtp_t *raop_rtp, const char *dacp_id, const char
 
     /* Set dacp stuff in thread instead */
     MUTEX_LOCK(raop_rtp->run_mutex);
+    if (raop_rtp->dacp_id) {
+      free(raop_rtp->dacp_id);
+    }
     raop_rtp->dacp_id = strdup(dacp_id);
+    if (raop_rtp->active_remote_header) {
+      free(raop_rtp->active_remote_header);
+    }
     raop_rtp->active_remote_header = strdup(active_remote_header);
     MUTEX_UNLOCK(raop_rtp->run_mutex);
 }
