@@ -18,7 +18,6 @@
  */
 
 #include "video_renderer.h"
-#include <assert.h>
 #include <gst/gst.h>
 #include <gst/app/gstappsrc.h>
 
@@ -95,6 +94,18 @@ static void append_videoflip (GString *launch, const videoflip_t *flip, const vi
 static video_renderer_t *renderer = NULL;
 static logger_t *logger = NULL;
 static unsigned short width, height, width_source, height_source;  /* not currently used */
+static bool first_packet = false;
+
+/* apple uses colorimetry=1:3:5:1 (not recognized by gstreamer v4l2)  *
+ * See .../gst-libs/gst/video/video-color.h in gst-plugins-base  *
+ * range = 1   -> GST_VIDEO_COLOR_RANGE_0_255      ("full RGB")  * 
+ * matrix = 3  -> GST_VIDEO_COLOR_MATRIX_BT709                   *
+ * transfer = 5 -> GST_VIDEO_TRANSFER_BT709                      *
+ * primaries = 1 -> GST_VIDEO_COLOR_PRIMARIES_BT709              *
+ * closest is BT709, 2:3:5:1 with                                *
+ * range = 2 -> GST_VIDEO_COLOR_RANGE_16_235 ("limited RGB")     */  
+
+static const char h264_caps[]="video/x-h264,stream-format=(string)byte-stream,alignment=(string)au";
 
 void video_renderer_size(float *f_width_source, float *f_height_source, float *f_width, float *f_height) {
     width_source = (unsigned short) *f_width_source;
@@ -104,8 +115,10 @@ void video_renderer_size(float *f_width_source, float *f_height_source, float *f
     logger_log(logger, LOGGER_DEBUG, "begin video stream wxh = %dx%d; source %dx%d", width, height, width_source, height_source);
 }
 
-void  video_renderer_init(logger_t *render_logger, const char *server_name, videoflip_t videoflip[2], const char *decoder, const char *videosink) {
+void  video_renderer_init(logger_t *render_logger, const char *server_name, videoflip_t videoflip[2], const char *parser,
+                          const char *decoder, const char *converter, const char *videosink) {
     GError *error = NULL;
+    GstCaps *caps = NULL;
     logger = render_logger;
 
     /* this call to g_set_application_name makes server_name appear in the  X11 display window title bar, */
@@ -116,24 +129,36 @@ void  video_renderer_init(logger_t *render_logger, const char *server_name, vide
     appname = NULL;
 
     renderer = calloc(1, sizeof(video_renderer_t));
-    assert(renderer);
+    g_assert(renderer);
 
-    gst_init(NULL,NULL);
-
-    GString *launch = g_string_new("appsrc name=video_source stream-type=0 format=GST_FORMAT_TIME is-live=true ! queue ! ");
+    GString *launch = g_string_new("appsrc name=video_source ! ");
+    g_string_append(launch, "queue ! ");
+    g_string_append(launch, parser);
+    g_string_append(launch, " ! ");
     g_string_append(launch, decoder);
-    g_string_append(launch, " ! videoconvert ! ");
+    g_string_append(launch, " ! ");
+    g_string_append(launch, converter);
+    g_string_append(launch, " ! ");    
     append_videoflip(launch, &videoflip[0], &videoflip[1]);
     g_string_append(launch, videosink);
     g_string_append(launch, " name=video_sink sync=false");
-    renderer->pipeline = gst_parse_launch(launch->str,  &error);
+    logger_log(logger, LOGGER_DEBUG, "GStreamer video pipeline will be:\n\"%s\"", launch->str);
+    renderer->pipeline = gst_parse_launch(launch->str, &error);
+    if (error) {
+        g_error ("get_parse_launch error (video) :\n %s\n",error->message);
+        g_clear_error (&error);
+    }
     g_assert (renderer->pipeline);
     g_string_free(launch, TRUE);
 
     renderer->appsrc = gst_bin_get_by_name (GST_BIN (renderer->pipeline), "video_source");
-    assert(renderer->appsrc);
+    g_assert(renderer->appsrc);
+    caps = gst_caps_from_string(h264_caps);
+    g_object_set(renderer->appsrc, "caps", caps, "stream-type", 0, "is-live", TRUE, "format", GST_FORMAT_TIME, NULL);
+    gst_caps_unref(caps);
+
     renderer->sink = gst_bin_get_by_name (GST_BIN (renderer->pipeline), "video_sink");
-    assert(renderer->sink);
+    g_assert(renderer->sink);
 
 #ifdef X_DISPLAY_FIX
     renderer->server_name = server_name;
@@ -146,7 +171,7 @@ void  video_renderer_init(logger_t *render_logger, const char *server_name, vide
     }
     if (x_display_fix) {
         renderer->gst_window = calloc(1, sizeof(X11_Window_t));
-        assert(renderer->gst_window);
+        g_assert(renderer->gst_window);
         get_X11_Display(renderer->gst_window);
     }
 #endif
@@ -166,19 +191,26 @@ void  video_renderer_init(logger_t *render_logger, const char *server_name, vide
 void video_renderer_start() {
     gst_element_set_state (renderer->pipeline, GST_STATE_PLAYING);
     renderer->bus = gst_element_get_bus(renderer->pipeline);
+    first_packet = true;
 }
 
-void video_renderer_render_buffer(raop_ntp_t *ntp, unsigned char* data, int data_len, uint64_t pts, int type) {
+void video_renderer_render_buffer(raop_ntp_t *ntp, unsigned char* data, int data_len, uint64_t pts, int nal_count) {
     GstBuffer *buffer;
-    assert(data_len != 0);
-    /* first four bytes of valid video data are 0x0, 0x0, 0x0, 0x1 */
-    /* first byte of invalid data (decryption failed) is 0x1 */
+    g_assert(data_len != 0);
+    /* first four bytes of valid  h264  video data are 0x00, 0x00, 0x00, 0x01.    *
+     * nal_count is the number of NAL units in the data: short SPS, PPS, SEI NALs *
+     * may  precede a VCL NAL. Each NAL starts with 0x00 0x00 0x00 0x01 and is    *
+     * byte-aligned: the first byte of invalid data (decryption failed) is 0x01   */
     if (data[0]) {
         logger_log(logger, LOGGER_ERR, "*** ERROR decryption of video packet failed ");
     } else {
+        if (first_packet) {
+            logger_log(logger, LOGGER_INFO, "Begin streaming to GStreamer video pipeline");
+            first_packet = false;
+        }
         buffer = gst_buffer_new_and_alloc(data_len);
-        assert(buffer != NULL);
-        GST_BUFFER_DTS(buffer) = (GstClockTime)pts;
+        g_assert(buffer != NULL);
+        GST_BUFFER_PTS(buffer) = (GstClockTime) pts;
         gst_buffer_fill(buffer, 0, data, data_len);
         gst_app_src_push_buffer (GST_APP_SRC(renderer->appsrc), buffer);
 #ifdef X_DISPLAY_FIX
@@ -189,7 +221,7 @@ void video_renderer_render_buffer(raop_ntp_t *ntp, unsigned char* data, int data
     }
 }
 
-void video_renderer_flush(video_renderer_t *renderer) {
+void video_renderer_flush() {
 }
 
 void video_renderer_stop() {
@@ -234,7 +266,16 @@ gboolean gstreamer_pipeline_bus_callback(GstBus *bus, GstMessage *message, gpoin
         gboolean flushing;
         gst_message_parse_error (message, &err, &debug);
         logger_log(logger, LOGGER_INFO, "GStreamer error: %s", err->message);
-        g_error_free (err);
+        if (strstr(err->message,"Internal data stream error")) {
+            logger_log(logger, LOGGER_INFO,
+                     "*** This is a generic GStreamer error that usually means that GStreamer\n"
+                     "*** was unable to construct a working video pipeline.\n"
+                     "*** If you are letting the default autovideosink select the videosink,\n"
+                     "*** GStreamer may be trying to use non-functional hardware h264 video decoding.\n"
+                     "*** Try using option -avdec to force software decoding or use -vs <videosink>\n"
+                     "*** to select a videosink of your choice (see \"man uxplay\")");
+        }
+	g_error_free (err);
         g_free (debug);
         gst_app_src_end_of_stream (GST_APP_SRC(renderer->appsrc));
 	flushing = TRUE;
